
#include "korners_engine_thread.h"
#include "korners_game_engine_ai.h"
#include "korners_game_engine_network.h"
#include "korners_scene.h"

#include <KDebug>


KornersEngineThread::KornersEngineThread( KornersScene* sc) : QThread(sc)
{
    curGameEngine = 0;
    sType = none;
    m_scene = sc;
}

KornersEngineThread::~KornersEngineThread()
{
    if(curGameEngine != 0)
    {
        delete curGameEngine;
        curGameEngine = 0;
    }
    quit();
    wait();
}

void KornersEngineThread::run()
{
    if(curGameEngine != 0)
    {
        kDebug() << "deleting cur game engine";
        delete curGameEngine;
    }

    switch(sType)
    {
        case ai:
            // Возможно можно не удалять тута
            kDebug() << "creating game engine ai";
            curGameEngine = new KornersGameEngineAI();
            break;
        case network_clnt:
            //delete curGameEngine;
            kDebug() << "creating game engine network client";
            curGameEngine = new KornersGameEngineNetwork(   startParams[0].toString(),
                                                            false,
                                                            startParams[1].toInt(), 
                                                            startParams[2].toString() );
            break;
        case network_serv:
            //delete curGameEngine;
            kDebug() << "creating game engine network server";
            curGameEngine = new KornersGameEngineNetwork(   startParams[0].toString(),
                                                            true,
                                                            startParams[1].toInt() );
            break;
        default:
            break;
    }

    if(curGameEngine != 0)
    {
        connect( m_scene, SIGNAL( playerMadeMove( QPoint , QPoint ) ), 
                 curGameEngine, SLOT( playersMove( QPoint , QPoint ) ) );

        connect( m_scene, SIGNAL( getMovesFromPoint(QPoint) ),
                 curGameEngine, SLOT( getAvailableMoves(QPoint) ) );

        connect( curGameEngine, SIGNAL( move(KornersMove) ),  
                 m_scene, SLOT( showMovePath(KornersMove) ) );

        connect( curGameEngine, SIGNAL( availableMoves( const QVector<QPoint>* ) ), 
                 m_scene, SLOT( showPaths( const QVector<QPoint>* ) ) );
        
        connect( curGameEngine, SIGNAL( engineMessage(QString) ), 
                 m_scene, SLOT( showEngineMess(QString) ) );

        connect( curGameEngine, SIGNAL( canPlay() ), m_scene, SLOT( canPlay() ) );

        connect( curGameEngine, SIGNAL( youWin() ), m_scene, SLOT( youWin() ) );
        connect( curGameEngine, SIGNAL( youLost() ), m_scene, SLOT( youLost() ) );

        if(( sType == network_clnt ) || ( sType == network_serv ))
        {
            connect( curGameEngine, SIGNAL( opponentName( QString ) ), 
                     m_scene, SLOT( opponentName(QString) ) );
        }
    }

    exec();
}

void KornersEngineThread::startGameEngineAI()
{
    sType = ai;
    restart();
}

void KornersEngineThread::startGameEngineNetwork(const QString& net_name, int port, bool server, const QString& ip_adress)
{
    startParams.append(net_name);
    startParams.append(port);
    if(!server)
    {
        startParams.append(ip_adress);
        sType = network_clnt;
    }
    else
        sType = network_serv;
   
    restart();
}
