
#include "korners_protocol.h"

#include <QPoint>
#include <KDebug>


KornersGameProtocol::KornersGameProtocol( const QString& name, bool server )
{
    m_version = 1;
    m_playerName = name;
    m_server = server;
}

void KornersGameProtocol::processMessage(const QByteArray& message)
{
    int messType = message[0];
    kDebug() << "message " << messType;
    KornersMessage msg;

    switch(messType)
    {
        case KornersMessage::handshake_s :
        case KornersMessage::handshake_c :
		{
		    // Можно ходить
		    msg.type = messType;
                    int protoVer = message[1];
                    if(protoVer == m_version)
                    {
                        QString name = message.right( message.length() - 2 );
                        msg.data.push_back(name);
                        emit(messageParsed(msg));
                    }
                    else
                    {
                        kDebug() << "Network protocol version doesn't match";
                    }
		}
                break;
        case KornersMessage::move :
		{
		    msg.type = KornersMessage::move;
                    QPoint from(message[1], message[2]);
                    QPoint to(message[3], message[4]);
                    msg.data.push_back(from);
                    msg.data.push_back(to);
                    emit(messageParsed(msg));
		}
                break;

        case KornersMessage::regret :
		{
		    msg.type = KornersMessage::regret;
		    emit(messageParsed(msg));
		}
                break;
        default:
            break;
    }
}


const QByteArray& KornersGameProtocol::start()
{
    static QByteArray msg;
    msg.clear();
    if( m_server )
        msg.append( KornersMessage::handshake_s );
    else
        msg.append( KornersMessage::handshake_c );
    msg.append(m_version);
    msg.append( m_playerName.toUtf8() );
    return msg;
}

const QByteArray& KornersGameProtocol::stop()
{
    static QByteArray msg;
    msg.clear();
    msg.append( KornersMessage::regret );
    return msg;
}

const QByteArray& KornersGameProtocol::move( const QPoint& src, const QPoint& dst )
{
    static QByteArray msg;
    msg.clear();
    msg.append( KornersMessage::move );
    msg.append( src.x() );
    msg.append( src.y() );
    msg.append( dst.x() );
    msg.append( dst.y() );
    return msg;
}